# DeepLab v3

## モデル

* [DeepLab v3](https://github.com/tensorflow/models/tree/master/research/deeplab)  
  * optimized_graph_voc.pb  
  * voc.names（サンプルプログラムに同梱）  
  * optimized_graph_cityscapes.pb  
  * cityscapes.names（サンプルプログラムに同梱）  

## 著作権/ライセンス

* Copyright (c) 2016 The TensorFlow Authors  
* [Apache License Version 2.0](https://github.com/tensorflow/models/blob/master/LICENSE)  

## 変換方法

1. TensorFlow 1.15をインストールする。  

  ```bash
  pip3 install tensorflow==1.15.5
  ```

2. TensorFlow Model ZooからDeepLab v3のモデルをダウンロードする。  
  ダウンロードした圧縮ファイル（deeplabv3_mnv2_pascal_trainval_2018_01_29.tar.gz、deeplabv3_mnv2_cityscapes_train_2018_02_05.tar.gz）を展開する。  

  * [mobilenetv2_coco_voc_trainval](http://download.tensorflow.org/models/deeplabv3_mnv2_pascal_trainval_2018_01_29.tar.gz)  
  * [mobilenetv2_coco_cityscapes_trainfine](http://download.tensorflow.org/models/deeplabv3_mnv2_cityscapes_train_2018_02_05.tar.gz)  

3. サンプルプログラムに同梱されている**optimize_voc.py**、**optimize_cityscapes.py** [^1]を実行してモデルを変換する。  

  * optimize_voc.py
  ```python
  import os
  import tensorflow as tf
  from tensorflow.tools.graph_transforms import TransformGraph

  def get_graphdef(path):
      tf.compat.v1.reset_default_graph
      with tf.python.ops.Graph().as_default():
          with tf.io.gfile.GFile(path, 'rb') as f:
              graphdef = tf.compat.v1.GraphDef()
              graphdef.ParseFromString(f.read())
              return graphdef

  def main()
      directory = "./"
      in_graph = os.path.join(directory, "deeplabv3_mnv2_pascal_trainval/frozen_inference_graph.pb")
      out_graph = os.path.join(directory, "optimized_graph_voc.pb")
      inputs = ["sub_7"]
      outputs = ["ResizeBilinear_3"]
      transforms = [
          "remove_nodes(op=Identity)",
          "merge_duplicate_nodes",
          "strip_unused_nodes",
          "fold_constants(ignore_errors=true)",
          "fold_batch_norms",
          "fold_old_batch_norms"
      ]
      graphdef = get_graphdef(in_graph)
      optimized_graphdef = TransformGraph(graphdef, inputs, outputs, transforms)
      tf.io.write_graph(optimized_graphdef, logdir=directory, as_text=False, name=out_graph)
      print("[success] transform optimize graph {0}".format(out_graph))

  if __name__ == '__main__':
      main()
  ```

  * optimize_cityscapes.py
  ```python
  import os
  import tensorflow as tf
  from tensorflow.tools.graph_transforms import TransformGraph

  def get_graphdef(path):
      tf.compat.v1.reset_default_graph
      with tf.python.ops.Graph().as_default():
          with tf.io.gfile.GFile(path, 'rb') as f:
              graphdef = tf.compat.v1.GraphDef()
              graphdef.ParseFromString(f.read())
              return graphdef

  def main()
      directory = "./"
      in_graph = os.path.join(directory, "deeplabv3_mnv2_cityscapes_train/frozen_inference_graph.pb")
      out_graph = os.path.join(directory, "optimized_graph_cityscapes.pb")
      inputs = ["sub_7"]
      outputs = ["ResizeBilinear_3"]
      transforms = [
          "remove_nodes(op=Identity)",
          "merge_duplicate_nodes",
          "strip_unused_nodes",
          "fold_constants(ignore_errors=true)",
          "fold_batch_norms",
          "fold_old_batch_norms"
      ]
      graphdef = get_graphdef(in_graph)
      optimized_graphdef = TransformGraph(graphdef, inputs, outputs, transforms)
      tf.io.write_graph(optimized_graphdef, logdir=directory, as_text=False, name=out_graph)
      print("[success] transform optimize graph {0}".format(out_graph))

  if __name__ == '__main__':
      main()
  ```

4. 変換が成功するとoptimized_graph_voc.pb、optimized_graph_cityscapes.pbのファイルが生成される。

## Notebook

* [deeplab-v3.ipynb | Google Colab](https://colab.research.google.com/drive/1oUl7tSVgUYgJHHkNg7zldQGRc-01HIUA?usp=sharing)  

---
[^1]: Ningxin Huさんが公開している[スクリプト](https://github.com/opencv/opencv/issues/17362#issuecomment-634407860)を参考にしました。  
